import { BaseSQLWorker } from './baseSQLWorker';
import { CancellationToken, editor, languages, Position, Uri } from './fillers/monaco-editor-core';
import type { LanguageServiceDefaults } from './monaco.contribution';
export interface WorkerAccessor<T extends BaseSQLWorker> {
    (...uris: Uri[]): Promise<T>;
}
export declare class DiagnosticsAdapter<T extends BaseSQLWorker> {
    private _languageId;
    private _worker;
    private readonly _defaults;
    private _disposables;
    private _listener;
    constructor(_languageId: string, _worker: WorkerAccessor<T>, _defaults: LanguageServiceDefaults);
    dispose(): void;
    private _doValidate;
}
export declare class CompletionAdapter<T extends BaseSQLWorker> implements languages.CompletionItemProvider {
    private readonly _worker;
    private readonly _defaults;
    constructor(_worker: WorkerAccessor<T>, _defaults: LanguageServiceDefaults);
    get triggerCharacters(): string[];
    provideCompletionItems(model: editor.IReadOnlyModel, position: Position, context: languages.CompletionContext, _token: CancellationToken): Promise<languages.CompletionList>;
}
/**
 * The adapter is for the definition of the symbol at the given position and document.
 **/
export declare class DefinitionAdapter<T extends BaseSQLWorker> implements languages.DefinitionProvider {
    private readonly _worker;
    private readonly _defaults;
    constructor(_worker: WorkerAccessor<T>, _defaults: LanguageServiceDefaults);
    /**
     * Provide the definition of the symbol at the given position and document.
     **/
    provideDefinition(model: editor.IReadOnlyModel, position: Position, _token: CancellationToken): languages.ProviderResult<languages.Definition | languages.LocationLink[]>;
}
/**
 * The adapter is for the references of the symbol at the given position and document.
 **/
export declare class ReferenceAdapter<T extends BaseSQLWorker> implements languages.ReferenceProvider {
    private readonly _worker;
    private readonly _defaults;
    constructor(_worker: WorkerAccessor<T>, _defaults: LanguageServiceDefaults);
    /**
     * Provide a set of project-wide references for the given position and document.
     **/
    provideReferences(model: editor.IReadOnlyModel, position: Position, _context: languages.ReferenceContext, _token: CancellationToken): languages.ProviderResult<languages.Location[]>;
}
